#!/opt/alt/python37/bin/python3 -bb
# -*- coding: utf-8 -*-

# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2019 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENSE.TXT

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import os
import sys
from shutil import copy

import cldetectlib as detect
import lvectllib
from clcommon import get_lve_version
from clcommon.utils import mod_makedirs
from clhooklib import log_error, check_cpanel_version


CPANEL_EXTENSIONS_DIR = '/var/cpanel/packages/extensions'
SOURCE_PATH = '/usr/share/l.v.e-manager/'
SCRIPT_NAME = 'install_lve_tmp.py'
VE_CFG_PATH = '/etc/container/ve.cfg'


def create_lve_default_settings():
    if os.path.isfile(VE_CFG_PATH):
        lvectllib.init()
        lvectllib.get_XML_cfg()
        try:
            file = open(CPANEL_EXTENSIONS_DIR + '/lve', 'w')
        except OSError:
            log_error(SCRIPT_NAME, 'Can\'t open LVE default setting file')
            sys.exit(1)
        file.write('_NAME=LVE Settings'+'\n')
        for i in list(lvectllib.ve_defaults) + ['inodes_soft', 'inodes_hard', 'mysql_cpu', 'mysql_io']:
            try:
                file.write('lve_' + i + '=DEFAULT\n')
            except IOError:
                log_error(SCRIPT_NAME, 'Can\'t write LVE settings to default extension file')
                sys.exit(1)
        file.close()


if detect.is_cpanel():
    if check_cpanel_version():
        LVE_VER = get_lve_version()
        if LVE_VER[1] != 'OK':
            log_error(SCRIPT_NAME, 'Can\'t detect LVE version')
            sys.exit(1)
        if not os.path.lexists(CPANEL_EXTENSIONS_DIR):
            mod_makedirs(CPANEL_EXTENSIONS_DIR, 0o700)
        if (LVE_VER[0] > 6) and os.path.isfile(SOURCE_PATH + 'cpanel/extension/lve.tt2'):
            copy(SOURCE_PATH + 'cpanel/extension/lve.tt2', CPANEL_EXTENSIONS_DIR + '/lve.tt2')
        create_lve_default_settings()
