<?php
/**
 * Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2019 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * http://cloudlinux.com/docs/LICENSE.TXT
 */

class Modules_PleskLvemanager_Controllers_Spa extends Modules_PleskLvemanager_Controllers_Base
{
    const OWNER_ADMIN = 'admin';
    const OWNER_USER = 'user';
    const OWNER_RESELLER = 'reseller';

    /**
     * Path to "cloudlinux-cli.py" script.
     *
     * @var string
     */
    public $cloudlinuxCLI = '/usr/bin/sudo /usr/share/l.v.e-manager/utils/cloudlinux-cli.py';
    public $cloudlinuxCLIUser = '/usr/share/l.v.e-manager/utils/cloudlinux-cli-user.py';

    protected $owner = 'user';

    /**
     * Required parameters.
     *
     * @var array
     */
    public $userInfo;

    public function init()
    {
        parent::init();
        if ($this->client->isClient() || $this->session->isImpersonated()) {
            $this->clientId = $this->client->isClient()
                ? $this->client->getId()
                : $this->session->getImpersonatedClientId();
        }
    }

    /**
     * Formatting of userInfo.
     *
     * @throws pm_Exception
     */
    protected function prepareUserInfo()
    {
        $this->userInfo = array(
            'username' => $this->getLogin(),
            'lve-id' => $this->getUID(),
        );
    }

    /**
     * Processes of incoming post request.
     *
     * @param $owner
     * @param null $pluginName - passed by user request controllers
     * @throws pm_Exception
     */
    public function processRequest($owner, $pluginName = NULL)
    {
        $this->_checkVulnerabilities();

        $this->owner = $owner;

        if (!isset($_POST['command'])) {
            $this->sendErrorResponse('COMMAND NOT SPECIFIED');
        }

        $data['owner'] = $this->owner;

        if (isset($pluginName)) {
            $data['plugin_name'] = $pluginName;
        }

        $data['command'] = $_POST['command'];

        if (isset($_POST['method'])) {
            $data['method'] = $_POST['method'];
        }

        if (isset($_POST['params'])) {
            $data['params'] = $_POST['params'];
        }
        if (!empty($_FILES)) {
            $fileList = array();
            // check for uploaded files
            $files = $_FILES['attachments']['tmp_name'];
            foreach( $files as $fileNum => $tempName ) {
                if( !empty($tempName) && is_uploaded_file( $tempName )) {
                    $fileList[] = array(
                        'file' => $tempName,
                        'name' => $_FILES['attachments']['name'][$fileNum]
                    );
                }
            }
            if (!empty($fileList)) {
                $data['attachments'] = $fileList;
            }
        }

        if (in_array($this->owner, array(self::OWNER_USER, self::OWNER_RESELLER))) {
            $this->prepareUserInfo();
            $data['user_info'] = $this->userInfo;
        }

        if (isset($_POST['mockJson'])) {
            $data['mockJson'] = $_POST['mockJson'];
        }
        // TODO: LVEMAN-1767, remove this `if`
        if (
            (array_key_exists('params', $data) &&
            (isset($data['params']['interpreter']) && $data['params']['interpreter'] == 'php')) ||
            ($data['command'] == 'cloudlinux-statistics') ||
            ($data['command'] == 'cloudlinux-quota') ||
            ($data['command'] == 'cloudlinux-top') ||
            ($data['command'] == 'cloudlinux-snapshots') ||
            ($data['command'] == 'cloudlinux-charts') ||
            ($data['command'] == 'cloudlinux-statsnotifier') ||
            ($data['command'] == 'spa-get-user-info') ||
            ($data['command'] == 'spa-get-domains') ||
            ($data['command'] == 'cloudlinux-awp-user') ||
            ($data['command'] == 'cl-smart-advice-user') ||
            ($data['command'] == 'cloudlinux-xray-user-manager')
        ) {
            // run without cagefs
            $this->cloudlinuxUserCLI = sprintf('/usr/bin/sudo -u %s %s', $this->getLogin(), $this->cloudlinuxCLIUser);
        } else {
            // enter to cagefs
            $this->cloudlinuxUserCLI = $this->cloudlinuxCLIUser;
        }

        $fullCommandStr = sprintf(
            "%s --data=%s 2>&1",
            $this->owner == 'user' ? $this->cloudlinuxUserCLI : $this->cloudlinuxCLI, base64_encode(json_encode($data))
        );

        // Only for testing. Remove this lines with echo on working system!!!
        //echo 'json: '.json_encode($data)."\n";
        //echo 'command: '.$fullCommandStr."\n";
        putenv('LC_ALL=en_US.UTF-8');

        ob_start();
        passthru($fullCommandStr);
        $responseInJson = ob_get_contents();
        ob_end_clean();

        $response = json_decode($responseInJson, true);

        if (is_null($response) && !empty($responseInJson)) {
            $this->sendErrorResponse(
                'ERROR.wrong_received_data', false, false,
                503, $responseInJson
            );
        }
        if (isset($response['result']) && $response['result'] === 'file') {
            $this->serveFile($response['filepath'], $response['filesize'], $data);
        } else if (isset($response['result']) && $response['result'] !== 'success' && $response['result'] !== 'rollback') {
            $this->sendErrorResponse($responseInJson, true);
        } else if (empty($responseInJson)) {
            $this->sendErrorResponse('RESPONSE OF COMMAND IS EMPTY');
        }

        echo $responseInJson;
    }

    /**
     * Returns response to SPA with error code, message and break execution of
     * script with status code.
     *
     * @param $message
     * @param $isJSON
     * @param $logoutSignal
     * @param int $statusCode
     * @param string $details
     */
    public function sendErrorResponse(
        $message, $isJSON = false,
        $logoutSignal = false, $statusCode = 503,
        $details = ''
    ) {
        http_response_code($statusCode);

        if ($isJSON) {
            echo $message;
        } else {
            $response = array(
                'result' => $message,
                'logoutSignal' => $logoutSignal,
                'details' => $details,
            );

            echo json_encode($response);
        }

        exit();
    }

    /**
     * Returns response to SPA with unavailable page and break execution of
     * script with status code.
     *
     * @param $message
     * @param $icon
     * @param $statusCode
     */
    public function unavailableError(
        $message, $context,
        $icon = 'info',
        $statusCode = 503
    ) {
        $response = array(
            'result' => 'error',
            'context' => $context,
            'error_id' => null,
            'code' => $statusCode,
            'error_id' => $message,
            'icon' => $icon
        );
        http_response_code($statusCode);
        echo json_encode($response);
        exit();
    }

    public function serveFile($fileName, $fileSize, $requestData) {
        header("Content-Type: application/x-download");
        header("Content-Length: $fileSize");
        $requestData['method'] = 'log_data';
        $fullCommandStr = sprintf(
            "%s --data=%s 2>&1",
            $this->owner == 'user' ? $this->cloudlinuxCLIUser : $this->cloudlinuxCLI, base64_encode(json_encode($requestData))
        );
        $descriptorspec = array(
           0 => array("pipe", "r"),   // stdin is a pipe that the child will read from
           1 => array("pipe", "w"),   // stdout is a pipe that the child will write to
           2 => array("pipe", "w")    // stderr is a pipe that the child will write to
        );
        $pipes = array();
        $process = proc_open($fullCommandStr, $descriptorspec, $pipes);
        if (is_resource($process)) {
            while ($s = fgets($pipes[1])) {
                print $s;
                flush();
            }
        }
        exit();
    }

    private function _checkVulnerabilities()
    {
        $this->_checkCSRFToken();
        $this->_checkReferer();
    }

    private function _checkCSRFToken()
    {
        if (!isset($_COOKIE['csrftoken']) || $_COOKIE['csrftoken'] !== $_SERVER['HTTP_X_CSRFTOKEN']) {
            if (empty($_SERVER['HTTPS'])) {
                $toUrl = str_replace(['http', $_SERVER['SERVER_PORT']], ['https','8443'], $_SERVER['HTTP_ORIGIN']);
                $this->unavailableError('Please use secure URL <a href="{url}">{url}</a>', array('url' => $toUrl));
            } else {
                $this->sendErrorResponse('BAD FORGERY PROTECTION TOKEN', false, true);
            }
        }
    }

    private function _checkReferer()
    {
        if (!preg_match(
            sprintf(
                '/^%s:\/\/%s/',
                (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on')
                    ? 'https' : 'http',
                $_SERVER['HTTP_HOST']
            ),
            $_SERVER['HTTP_REFERER'])
        ) {
            $this->sendErrorResponse('BAD REFERER', false, true);
        }
    }
}
