<?php
/**
 * Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2019 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * http://cloudlinux.com/docs/LICENSE.TXT
 */

class Modules_PleskLvemanager_DataSource
{
    private $_username;
    private $_host;
    private $_password;
    private $_database;
    private $_dbh;

    public function __construct()
    {
        $this->_username = 'admin';
        $this->_host = 'localhost';
        $this->_password = $this->getPassword();
        $this->_database = 'psa';
        $this->_dbh = $this->getDBHandler();
    }

    private function getPassword()
    {
        $filename = "/etc/psa/.psa.shadow";

        if (is_readable($filename)) {
            return trim(file_get_contents($filename));
        }

        return null;
    }

    private function getDBHandler()
    {
        $dbh = new mysqli(
            $this->_host,
            $this->_username,
            $this->_password,
            $this->_database
        );

        if (mysqli_connect_errno()) {
            exit();
        }

        return $dbh;
    }

    function get_user_by_id_and_domainId($id, $domainId)
    {
        $query = "SELECT DISTINCT sys_users.login FROM domains, hosting, sys_users WHERE domains.cl_id=? AND domains.id=? AND domains.id=hosting.dom_id AND hosting.sys_user_id=sys_users.id";

        if ($sth = $this->_dbh->prepare($query)) {
            $entries = array();
            $sth->bind_param("ss", $id, $domainId);
            $sth->execute();
            $sth->bind_result($rv);

            while ($sth->fetch()) {
                $entries[] = $rv;
            }

            $sth->close();

            return $entries;
        }

        return null;
    }

    public function get_id_by_impersonated_user($username)
    {
        $query = "SELECT domains.cl_id FROM domains, hosting, sys_users WHERE sys_users.id=hosting.sys_user_id AND hosting.dom_id=domains.id AND sys_users.login=?";

        if ($sth = $this->_dbh->prepare($query)) {
            $sth->bind_param("s", $username);
            $sth->execute();
            $sth->bind_result($rv);
            $sth->fetch();
            $sth->close();

            return $rv;
        }

        return null;
    }

    public function endSession()
    {
        $this->_dbh->close();
    }
}
